@@ -6,7 +6,9 @@
 //!
 //! ## Usage
 //!
-//! There are two methods you need to be familiar with to get started: `Menu::new` which takes an `Iterator` as an argument, and `Menu::show` which initializes `ncurses` and displays the menu.
+//! There are two methods you need to be familiar with to get started:
+//! `Menu::new` which takes an `Iterator` as an argument, and `Menu::show`
+//! which initializes `ncurses` and displays the menu.
 //!
 //! Here is a minimal example that displays the range  `0..100` in a menu:
 //!
@@ -21,11 +23,15 @@
 //! }
 //! ```
 //!
-//! Either `↓↑` or `jk` can be used to scroll, and `return` is used to select. `ESC` or `q` can be used to quit.
+//! ![basic config](https://raw.githubusercontent.com/nathom/youchoose/main/screenshots/basic.png)
+//!
+//! Either `↓↑` or `jk` can be used to scroll, and `return` is used to select.
+//! `ESC` or `q` can be used to quit.
 //!
 //! **Previews**
 //!
-//! The `youchoose::Menu` has a preview feature, which executes a command and shows the results on a seperate pane.
+//! The `youchoose::Menu` has a preview feature, which executes a command and
+//! shows the results on a seperate pane.
 //!
 //! ```rust
 //! use youchoose;
@@ -49,6 +55,8 @@
 //! }
 //! ```
 //!
+//! ![preview](https://raw.githubusercontent.com/nathom/youchoose/main/screenshots/with_preview.png)
+//!
 //! **Customization**
 //!
 //! Let's take a look at an example that showcases the available methods for customization.
@@ -59,7 +67,7 @@
 //! fn main() {
 //!     let mut menu = youchoose::Menu::new(0..100)
 //!         .preview(multiples)              // Sets the preview function
-//!         .preview_pos(youchoose::ScreenSide::Bottom)  // Sets the position of the preview pane
+//!         .preview_pos(youchoose::ScreenSide::Bottom, 0.3)  // Sets the position of the preview pane
 //!         .preview_label(" multiples ".to_string())    // Sets the text at the top of the preview pane
 //!         .multiselect()                   // Allows multiple items to be selected
 //!         .icon(":(")                      // Sets the default (not selected) icon for an item
@@ -75,8 +83,9 @@
 //! fn multiples(num: i32) -> String {
 //!     // --- Snip ---
 //! }
-//!
 //! ```
+//!
+//! ![fully customized](https://raw.githubusercontent.com/nathom/youchoose/main/screenshots/customized.png)
 
 use std::fmt;
 // use std::fs::OpenOptions;
@@ -86,6 +95,7 @@ use std::ops;
 
 use ncurses::*;
 
+/// A Menu that lazily displays an iterable and (optionally) its preview.
 pub struct Menu<'a, I, D>
 where
     D: fmt::Display,
@@ -115,6 +125,8 @@ where
     D: fmt::Display,
     I: Iterator<Item = D>,
 {
+    /// Create a new menu object. The iterable passed in must contain displayable
+    /// items.
     pub fn new(iter: I) -> Menu<'a, I, D> {
         let screen = Screen::new(ScreenSide::Full, 0.5);
 
@@ -146,6 +158,7 @@ where
         }
     }
 
+    /// Initialize curses and display the menu on the screen.
     pub fn show(&mut self) -> Vec<usize> {
         init_curses();
 
@@ -262,8 +275,21 @@ where
 
     fn multiselect_item(&mut self) -> RetCode {
         let curr_item_idx = self.state.start + self.state.hover;
-        self.state.items[curr_item_idx].select();
-        self.selection.push(curr_item_idx);
+        let curr_item = &mut self.state.items[curr_item_idx];
+        curr_item.select();
+
+        let item_idx_pos =
+            match self.selection.iter().position(|x| *x == curr_item_idx) {
+                Some(idx) => idx as i32,
+                None => -1,
+            };
+
+        if curr_item.chosen() && item_idx_pos == -1 {
+            self.selection.push(curr_item_idx);
+        } else if !curr_item.chosen() && item_idx_pos != -1 {
+            self.selection.remove(item_idx_pos as usize);
+        }
+
         Pass
     }
 
@@ -299,8 +325,9 @@ where
         Pass
     }
 
-    // CONFIG
-
+    /// Add a preview pane that displays the result of applying the function
+    /// passed in to each item in the iterable. The function must return a
+    /// String.
     pub fn preview<F>(mut self, func: F) -> Menu<'a, I, D>
     where
         F: Fn(D) -> String + 'static,
@@ -311,6 +338,13 @@ where
         self
     }
 
+    /// Sets the position of the preview pane. The `side` parameter determines
+    /// the side on which the pane sits. The `width` parameter is a float between
+    /// `0.0` and `1.0`, inclusive. It determines the proportion of the screen that
+    /// the preview pane should use.
+    ///
+    /// The menu's side is automatically switched to opposite the preview pane's side,
+    /// and the menu's width is set to `1 - width`.
     pub fn preview_pos(
         mut self,
         side: ScreenSide,
@@ -325,16 +359,20 @@ where
         self
     }
 
+    /// Sets the default icon of the menu. This is displayed before each entry.
     pub fn icon(mut self, icon: &'a str) -> Menu<'a, I, D> {
         self.item_icon = icon;
         self
     }
 
+    /// Sets the icon displayed when an item is selected in multiselect mode.
     pub fn selected_icon(mut self, icon: &'a str) -> Menu<'a, I, D> {
         self.chosen_item_icon = icon;
         self
     }
 
+    /// Sets the text displayed on top of the preview box. It is recommended to surround the label
+    /// with spaces for aesthetic reasons. If it is not set, `" preview "` will be used.
     pub fn preview_label(mut self, label: String) -> Menu<'a, I, D> {
         self.preview
             .as_mut()
@@ -343,26 +381,50 @@ where
         self
     }
 
+    /// Adds a keybinding that triggers a multiselection. This inputs an `ncurses` keycode.
+    /// All ascii keys can be set by passing the character as an `i32`. The keycodes for
+    /// special keys can be found by importing `ncurses` and using the provided constants
+    /// or by testing with the `getch` function. For example, running the following will display
+    /// the keycodes on the screen.
+    ///
+    /// ```
+    /// // Make sure to add ncurses as a dependency!
+    /// use ncurses::*;
+    /// fn main() {
+    ///     initscr();
+    ///     loop {
+    ///         let c: i32 = getch();
+    ///         clear();
+    ///         if c == 'q' as i32 {break}
+    ///         addstr(&format!("Pressed key with keycode {}!", c.to_string()));
+    ///     }
+    ///     endwin();
+    /// }
+    /// ```
     pub fn add_multiselect_key(mut self, key: i32) -> Menu<'a, I, D> {
         self.keys.multiselect.push(key);
         self
     }
 
+    /// Adds a keybinding that triggers an up movement. See [`add_multiselect_key`](struct.Menu.html#method.add_multiselect_key) for more information.
     pub fn add_up_key(mut self, key: i32) -> Menu<'a, I, D> {
         self.keys.up.push(key);
         self
     }
 
+    /// Adds a keybinding that triggers a down movement. See [`add_multiselect_key`](struct.Menu.html#method.add_multiselect_key) for more information.
     pub fn add_down_key(mut self, key: i32) -> Menu<'a, I, D> {
         self.keys.down.push(key);
         self
     }
 
+    /// Adds a keybinding that triggers a selection. See [`add_multiselect_key`](struct.Menu.html#method.add_multiselect_key) for more information.
     pub fn add_select_key(mut self, key: i32) -> Menu<'a, I, D> {
         self.keys.select.push(key);
         self
     }
 
+    /// Allow multiple items to be selected from the menu.
     pub fn multiselect(mut self) -> Menu<'a, I, D> {
         self.config.multiselect = true;
         self
@@ -683,12 +745,14 @@ impl fmt::Display for Pair {
     }
 }
 
+/// Determines the side on which a pane should be located.
 #[derive(Copy, Clone)]
 pub enum ScreenSide {
     Left,
     Right,
     Top,
     Bottom,
+    /// This option is not affected by width. It will always fill the screen.
     Full,
 }
 
@@ -848,7 +912,7 @@ where
 //     Ok(())
 // }
 
-pub fn init_curses() {
+fn init_curses() {
     // Allow unicode characters
     let locale_conf = LcCategory::all;
     setlocale(locale_conf, "en_US.UTF-8");
@@ -873,28 +937,6 @@ pub fn init_curses() {
     keypad(stdscr(), true);
 }
 
-pub fn end_curses() {
+fn end_curses() {
     endwin();
 }
-
-#[cfg(test)]
-mod tests {
-    use super::*;
-
-    #[test]
-    fn def_item_formatting() {
-        let icon = String::from("xx");
-        let chosen = String::from("--");
-        let mut item = Item::new("test string", &icon, &chosen);
-        assert_eq!(item.to_string(), "xx test string");
-    }
-
-    #[test]
-    fn chosen_item_fomatting() {
-        let icon = String::from("xx");
-        let chosen = String::from("--");
-        let mut item = Item::new("test string", &icon, &chosen);
-        item.select();
-        assert_eq!(item.to_string(), "-- test string");
-    }
-}
